@@ -90,7 +90,7 @@ class HaproxyConfig(object):
         proxy_ssl_apps = []
 
         for app in apps_list:
-            if not app['web_ip']:
+            if not 'web_ip' in app or not app['web_ip']:
                 continue
 
             domains = [app['fullname']]
